#
# Pearl routines common for the WebForum
# Copyright (C) 1994 Afzal Ballim
# afzal@divsun.unige.ch

############################################################
#
# WFHtmlHeader: print the content header
sub WFHtmlHeader {
    print "Content-type: text/html\n\n";
    return 1;
}

############################################################
#
# WFError: send back an error message
sub WFError {
 print "<hr><h2>***ERROR***</h2>\n";
 print @_;
 print "\n<hr>\n";
}


############################################################
#
# WFIncFile: print the contents of a file to the std. output
#            Arguments after the file name are pattern/string
#            pairs to be applied to the input of the file.
sub WFIncFile {
    local ($file)=shift @_;
    local ($rep_prog);
    local (*FILE);
    open(FILE,"<$file")||return 0;

    while (@_) {
     $rep_prog = $rep_prog . "s^" . &WFProtectSpecials(shift(@_)) . 
                '^' . &WFProtectSpecials(shift(@_)) . "^g;";
    }
    while (<FILE>) {
     eval $rep_prog;
     print;
    }
    close(FILE);
    return 1;
}

############################################################
#
# WFGetFile: retrieve the contents of a file and return
#            an associative array. The body of the array
#            can be retrieved by the key "body". If the
#            file contains a header, then the header
#            contains arg/val pairs (1 per line) with both
#            arg and value encoded. They are returned in the
#            array with unencoded arg as key, and unencoded
#            value as, well the value.
#            The same substitutions as with WFIncFile are
#            possible on the body.

sub WFGetFile {
 local (%gf);
 local ($file)=shift @_;
 local ($rep_prog);
 local ($arg,$value);
 local (*FILE);
 open(FILE,"<$file")||return 0;

 $_ = <FILE>;
 if (/^WF File Header/) {
  $_=<FILE>;
  while (!/^End Header/) {
   /^([^=]+)=(.+)\n/;
   $arg=&WFDecodeDatum($1);
   $value=&WFDecodeDatum($2);
   $gf{"$arg"}=$value;
   $_=<FILE>;
  }
 }

 while (@_) {
  $rep_prog = $rep_prog . "s^" . shift(@_) . '^' . shift(@_) . "^og;";
 }
 while (<FILE>) {
  eval $rep_prog;
  $gf{"body"} = $gf{"body"} . $_;
 }

 close(FILE);
 return %gf;
}

############################################################
#
# WFPutFile: A file is written using an associative array
#            as its basis. The "body" key gives a body to
#            be written. If there are other keys, then they
#            are written to a header.

sub WFPutFile {
 local ($file)=shift @_;
 local (%gf)=@_;

 local ($k);
 local (@kys)=keys %gf;
 local (*FILE);
 open(FILE,">$file")||return 0;

 if ($#kys >0) {
  print FILE "WF File Header\n";
  foreach $k (@kys) {
    next if $k eq 'body';
    print FILE &WFEncodeDatum($k),"=",&WFEncodeDatum($gf{"$k"}),"\n";
  }
  print FILE "End Header\n";
 }

 print FILE $gf{"body"};
 close(FILE);
 return 1;
}

############################################################
#
# WFEncodeDatum: encode a string so that it can be used in the
#               path-info field of a GET method call or sent
#               straight to a POST method script as std. input
#               This is a pretty extreme version which encodes
#               just about anything.
sub WFEncodeDatum {
 local ($_) = @_[0];
 # various characters into hex equivalents
 s/([^A-Za-z0-9 ])/sprintf("%%%.2x",ord($1))/eg;
 # now convert spaces into + signs
 s/ /+/g;
 return $_;
}

############################################################
#
# WFDecodeDatum: Decode an encoded data item
sub WFDecodeDatum {
 local ($_) =@_[0];
 # convert + to space
 s/\+/ /g;
 # convert hex values to their character
 s/%(..)/pack("c",hex($1))/ge;
 return $_;
}

############################################################
#
# WFEncodeData: encode an associate array into an encoded
#               string of the form used by WWW.
sub WFEncodeData {
 local ($encoded)="";
 local (%in) = @_;
 local (@k) = keys %in;
 local ($ak);

 foreach $ak (@k) {
    $encoded = $encoded . "$ak=" . &WFEncodeDatum($in{"$ak"}) . "&";
 }
 return $encoded;
}

############################################################
#
# WFDecodeData: decode a data string into an associative
#               array.
sub WFDecodeData {
 local (%data);
 local (@tmp) = split('&',@_[0]);
 local ($av, @avp);

 foreach $av (@tmp) {
  @avp=split('=',$av);
  $data{"@avp[0]"}=&WFDecodeDatum(@avp[1]);
 }
 return %data;
}


############################################################
#
# WFDecodePathInfo: get values from Path Info
sub WFDecodePathInfo {
 local (@tmp)=split('/',$ENV{PATH_INFO});
 return &WFDecodeData($tmp[$#tmp]);
}


############################################################
#
# WFReadParse: This is Steven E. Brenner's ReadParse routine
#              changed so that an associate array is returned
#              by the call. As the change is so small, it 
#              should be considered copyright S. E. Brenner.
sub WFReadParse {
  if (@_) {
    local (*in) = @_;
  }

  local ($i, $loc, $key, $val);

  # Read in text
  if ($ENV{'REQUEST_METHOD'} eq "GET") {
    $in = $ENV{'QUERY_STRING'};
  } elsif ($ENV{'REQUEST_METHOD'} eq "POST") {
    for ($i = 0; $i < $ENV{'CONTENT_LENGTH'}; $i++) {
      $in .= getc;
    }
  } 

  @in = split(/&/,$in);

  foreach $i (0 .. $#in) {
    # Convert plus's to spaces
    $in[$i] =~ s/\+/ /g;

    # Convert %XX from hex numbers to alphanumeric
    $in[$i] =~ s/%(..)/pack("c",hex($1))/ge;

    # Split into key and value.
    $loc = index($in[$i],"=");
    $key = substr($in[$i],0,$loc);
    $val = substr($in[$i],$loc+1);
    $in{$key} .= '\0' if (defined($in{$key})); # \0 is the multiple separator
    $in{$key} .= $val;
  }

  return %in;
}


############################################################
#
# WFReadDBFile: given the name of a user, get the details of
#               that user from the database file.
sub WFReadDBFile {
 local ($file) = &WFDBName(@_[0]);

 return &WFReadAVFile($file);
}

############################################################
#
# WFReadAVFile: given a filename for an AV file, read it.
#               The AV file has lines that represent arg=value
#               The arg and value have been encoded.
sub WFReadAVFile {
 local (%db);
 local (@line);
 local ($ln);
 local (*UFILE);
 local ($arg,$value);
 local ($file) = @_[0];

 if (-r $file) {
  open(UFILE,"<$file")||return 0;
 }
 else {return -1;}
 push(@line,$_) while <UFILE>;
 foreach $ln (@line) {
  $ln =~ /^([^=]+)=(.+)\n/;
  $arg=&WFDecodeDatum($1);
  $value=&WFDecodeDatum($2);
  $db{"$arg"}=$value;
 }
 close(UFILE);
 return %db;
}

############################################################
#
# WFWriteDBFile: given a user name and an associative array,
#   write a database file with arg=value lines
sub WFWriteDBFile {
 local ($user,%db) = @_;
 local ($file) = &WFDBName($user);
 return &WFWriteAVFile($file,%db);
}

############################################################
#
# WFWriteAVFile: write a file of arg=value pairs
#
sub WFWriteAVFile {
 local ($k);
 local ($file,%db)=@_;
 local (*UFILE);

 open(UFILE,">$file")|| return 0;
 foreach $k (keys %db) {
  print UFILE &WFEncodeDatum($k),"=",
            &WFEncodeDatum($db{"$k"}),"\n";
 }
 return 1;
}

############################################################
#
# WFDBName: given a user name, return the file that contains
#           the user's data
sub WFDBName {
 return "ABSWFDBASE" . '/' . @_[0];
}

############################################################
#
# WFUserAuthorised: is a user authorised to access a forum?
#           This is decided by looking at the control file
#           to determine authorised groups, then looking at
#           the group file to see if that user belongs to
#           one of those authorised groups.
#
sub WFUserAuthorised {
 local ($user,$file)=@_;
 if (!-r $file) {return 0};
 local (%control)=&WFReadAVFile($file);
 local ($groups)=$control{"groups"};
 if ($control{"groups"} !~ /[A-Za-z]/) {return 1;}
 local (*GFILE);

 open(GFILE,"<GROUPFILE");
 while (<GFILE>) {
  if (/^([^:]*).*[ :]$user[ \n]/ # the user is in a group
   && ($groups=~/^$1 / ||
       $groups=~/ $1$/ ||
       $groups=~/ $1 /)) {
  close (GFILE);return 1;}
 }
 close(GFILE);
 return 0;
}

############################################################
#
# WFUserInGroup: is a user in a group?
#
sub WFUserInGroup {
 local ($user,$group)=@_;
 local (*GFILE);

 open(GFILE,"<GROUPFILE");
 while (<GFILE>) {
  if (/^$group:$user[ \n]/ || /^$group:.* $user[ \n]/)
	{close (GFILE);return 1;}
 }
 close(GFILE);
 return 0;
}

############################################################
#
# WFReadPasswdFile:

sub WFReadPasswdFile {
 local (*PWF);
 open(PWF,"<PASSWDFILE") || die "couldn't open password file\n";
 while (<PWF>) {
  /^([^:]*):(.*)\n$/;
  $PWF{$1}=$2;
 }
 close(PWF);

 return %PWF;
}

############################################################
#
# WFWritePasswdFile:

sub WFWritePasswdFile {
 local (%pwds)=@_;
 local ($pwd);
 local (*PWF);
 open(PWF,">PASSWDFILE") || die "couldn't open password file\n";
 foreach $pwd (sort keys(%pwds)) {
    print PWF "$pwd:$pwds{$pwd}\n";
 }
 close(PWF);
}

############################################################
#
# WFReadGroupFile: (doesn't this look familiar?)

sub WFReadGroupFile {
 local (*GRF);
 open(GRF,"<GROUPFILE") || die "couldn't open group file\n";
 while (<GRF>) {
  /^([^:]*): *(.*)\n$/;
  $GRF{$1}=$2;
 }
 close(GRF);

 return %GRF;
}

############################################################
#
# WFWriteGroupFile:

sub WFWriteGroupFile {
 local (%grps)=@_;
 local ($grp);
 local (*GRF);
 open(GRF,">GROUPFILE") || die "couldn't open group file\n";
 foreach $grp (sort keys(%grps)) {
    print GRF "$grp:$grps{$grp}\n";
 }
 close(GRF);
}

############################################################
#
# WFListNewRegs: return a list of new registered users.
sub WFListNewRegs {
 local (*users, @ausers, @nusers, $u);
 opendir(DIR,"ABSWFDBASE");
 @ausers=readdir(DIR);
 close(DIR);
 %users=&WFReadPasswdFile;
 foreach $u (@ausers) {
    next if $u eq '.';
    next if $u eq '..';
    next if $u eq 'admin.html';
    next if $u eq 'wfroot';
    next if $u eq 'ACCESSFILE';
    next if 'PASSWDFILE' =~ /$u/;
    next if 'GROUPFILE' =~ /$u/;
    next if $users{$u} ne '';
    push(@nusers,$u);
 }
 return sort @nusers;
}

############################################################
#
# WFProtectSpecials
#
# protect special things in a string
sub WFProtectSpecials {
 local ($s) = @_;
 $s =~ s/\$/\\\$/g;
 $s =~ s/\^/\\\^/g;
 $s =~ s/\&/\\\&/g;
 $s =~ s/\|/\\\|/g;
 $s =~ s/\"/\\\"/g;
 $s =~ s/\*/\\\*/g;
 $s =~ s/\?/\\\?/g;
 $s =~ s/\'/\\\'/g;
 $s =~ s/\ /\\\ /g;
 $s =~ s/\@/\\\@/g;
 $s =~ s/\#/\\\#/g;
 $s =~ s/\%/\\\%/g;
 return $s;
}

############################################################
#
# WFWrapHTMLCodes: protect > < & and "
sub WFWrapHTMLCodes {
 local ($string)=@_;
 $string =~ s^&^&amp\;^g;
 $string =~ s^<^&lt\;^g;
 $string =~ s^>^&gt\;^g;
 $string =~ s^"^&quot\;^g;
 return $string;
}

############################################################
#
# WFUnwrapHTMLCodes: unprotect > < & and "
sub WFUnwrapHTMLCodes {
 local ($string)=@_;
 $string =~ s^&lt\;^<^g;
 $string =~ s^&gt\;^>^g;
 $string =~ s^&quot\;^"^g;
 $string =~ s^&amp\;^&^g;
 return $string;
}

############################################################
#
# WFDateToNumber: convert a ctime date into a number
sub WFDateToNumber {
 local ($date) =@_;$date=~s/\n//g;
 local (@date) =split(' ',$date);
 local ($day,$month,$year)= (@date[2],@date[1],@date[4]);
 FOO: {
    $month=0, last FOO if $month eq "Jan";
    $month=1, $day+=31, last FOO if $month eq "Feb";
    $month=2, $day+=59, last FOO if $month eq "Mar";
    $month=3, $day+=90, last FOO if $month eq "Apr";
    $month=4, $day+=120, last FOO if $month eq "May";
    $month=5, $day+=151, last FOO if $month eq "Jun";
    $month=6, $day+=181, last FOO if $month eq "Jul";
    $month=7, $day+=212, last FOO if $month eq "Aug";
    $month=8, $day+=243, last FOO if $month eq "Sep";
    $month=9, $day+=273, last FOO if $month eq "Oct";
    $month=10, $day+=304, last FOO if $month eq "Nov";
    $month=11, $day+=334, last FOO if $month eq "Dec";
 }
 return $day+(365*$year);
}

1;#return true
